# This program implements the formulaic sequence identification described in
#
# Julian Brooke, Adam Hammond, David Jacob, Vivian Tsang, Graeme Hirst, and Fraser Shein
# Building Lexicons of Formulaic Language for Language Learners
# Proceedings of MWE 2015.
#
# Run the program without arguments to see a list of options
#
# This program expects a tokenized and tagged corpus with each
# line consisting of a token and a (POS) tag, separated by a tab
# and the SENT part of speech tag indicating the end of a sentence
# For best results, the tokens should be lemmatized and lower_cased
# and the POS tag should have any inflectional information stripped off
# (i.e., NNS -> NN). For a differently formatted corpora (or to do
# lemmatization, etc. on the fly) modify read_sentence_from_corpus
# in corpus_reader.py
# This version of the program has support for English and French, for
# other languages, modify pos_helper.py
# The number of workers (parallel processes) is fixed on the command line,
# default 1, but for large corpora more is recommended. It might be useful
# to set different numbers of workers for the different steps of the processes
# depending on available memory, since some parts of the process use more memory
# than others.
# For memory reasons (to ensure that memory is freed at the end of each step),
# this script executes each of the major steps in an entirely separate python
# process. If your python 2.7 interpreter is not accessible as "python",
# change py_exe below.
# Large temporary files are created at each step of the process which
# are deleted at the end. If the program crashes due to lack of memory,
# these temporary files can be used to resume the process
# Note that the once per x tokens threshold should be chosen so that the minimum
# frequency is greater than 1 (and perferably much greater than 1)
# The default is what was used in the paper (at least once per 10 million tokens)
##########################################################################


from optparse import OptionParser
from subprocess import call
import cPickle
import sys
import os


py_exe = "python"


            

if __name__ == "__main__":
    parser = OptionParser()
    parser.add_option("-c","--corpus",dest="corpus",help="the corpus to be segmented, should be tokenized with one sentence per line")
    parser.add_option("-w","--workers",default=1,dest="workers",type="int", help="the number of worker processes used, improves speed but increases memory usage by a factor of w")
    parser.add_option("-n",dest="n",default=8,type="int",help="largest n considered for n-grams, default 8")
    parser.add_option("-f","--frequency",dest="frequency",type="int",default=10000000, help="lowest allowed frequency for output vocab, once per FREQUENCY, default 10000000")
    parser.add_option("-q","--quiet",default=False,dest="silent",action="store_true",help="don't show progress")
    parser.add_option("-s","--sentences",type="int",dest="sentences",default=-1, help="Limit the number of sentences of the corpus processed, useful for testing")
    parser.add_option("-o","--output",default="vocab.txt", dest="output",help="the output multiword vocabulary (with corpus counts), default vocab.txt")
    parser.add_option("-l","--language",dest="lang",default="en",help="the language, for aspects relevant to part of speech (in particular, the syntactic definition of a gap). In current version, en and fr are supported options")
    options,arguments = parser.parse_args()
    fout = open("temp_options.dat","wb")
    cPickle.dump(options,fout,-1)
    fout.close()
    if not options.corpus:
        parser.print_help()
    else:
        if not options.silent:
            print "getting initial n-gram statistics"
        retcode = call([py_exe,"get_ngram_statistics.py"])
        if retcode != 0:
            print "ngram statistics gathering failed"
            sys.exit()
        if not options.silent:
            print "finding best POS for ngrams"
        retcode = call([py_exe,"get_best_POS.py"])
        if retcode != 0:
            print "best POS identification failed"
            sys.exit()            
        if not options.silent:
            print "getting LPR conditional probability statistics"
            print "(this could take a while)"
        retcode = call([py_exe,"get_LPR_statistics.py"])
        if retcode != 0:
            print "POS statistics gathering failed"
            sys.exit()
        if not options.silent:
            print "doing initial segmentation"
        retcode = call([py_exe,"LPR_ngram_seg.py"])
        if retcode != 0:
            print "segmentation failed"
            sys.exit()
        if not options.silent:
            print "doing vocabulary decomposition"
        retcode = call([py_exe,"LPR_vocab_decomp.py"])
        if retcode != 0:
            print "vocab decomposition failed"
            sys.exit()
        os.remove("temp_options.dat")
        os.remove("temp_ngrams.dat")
        os.remove("temp_best_POS.dat")
        os.remove("temp_LPR_stats.dat")
        os.remove("temp_initial_vocab.dat")

    
        
        
